@@ -27,14 +27,11 @@
 #define YT_USERNAME "GDataTest"
 #define YT_PASSWORD "gdata"
 
-/* TODO: probably a better way to do this; some kind of data associated with the test suite? */
-static GDataService *service = NULL;
-static GMainLoop *main_loop = NULL;
-
 static void
 test_authentication (void)
 {
 	gboolean retval;
+	GDataService *service;
 	GError *error = NULL;
 
 	/* Create a service */
@@ -56,10 +53,12 @@ test_authentication (void)
 	g_assert_cmpstr (gdata_service_get_username (service), ==, YT_USERNAME "@gmail.com");
 	g_assert_cmpstr (gdata_service_get_password (service), ==, YT_PASSWORD);
 	g_assert_cmpstr (gdata_youtube_service_get_youtube_user (GDATA_YOUTUBE_SERVICE (service)), ==, YT_USERNAME);
+
+	g_object_unref (service);
 }
 
 static void
-test_authentication_async_cb (GDataService *service, GAsyncResult *async_result, gpointer user_data)
+test_authentication_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
 {
 	gboolean retval;
 	GError *error = NULL;
@@ -81,27 +80,29 @@ test_authentication_async_cb (GDataService *service, GAsyncResult *async_result,
 static void
 test_authentication_async (void)
 {
+	GMainLoop *main_loop;
+	GDataService *service;
+
 	/* Create a service */
 	service = GDATA_SERVICE (gdata_youtube_service_new (DEVELOPER_KEY, CLIENT_ID));
 
 	g_assert (service != NULL);
 	g_assert (GDATA_IS_SERVICE (service));
 
-	gdata_service_authenticate_async (service, YT_USERNAME, YT_PASSWORD, NULL, (GAsyncReadyCallback) test_authentication_async_cb, NULL);
-
 	main_loop = g_main_loop_new (NULL, TRUE);
+	gdata_service_authenticate_async (service, YT_USERNAME, YT_PASSWORD, NULL, (GAsyncReadyCallback) test_authentication_async_cb, main_loop);
+
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
+	g_object_unref (service);
 }
 
 static void
-test_query_standard_feed (void)
+test_query_standard_feed (GDataService *service)
 {
 	GDataFeed *feed;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
 	feed = gdata_youtube_service_query_standard_feed (GDATA_YOUTUBE_SERVICE (service), GDATA_YOUTUBE_TOP_RATED_FEED, NULL, NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_FEED (feed));
@@ -113,7 +114,7 @@ test_query_standard_feed (void)
 }
 
 static void
-test_query_standard_feed_async_cb (GDataService *service, GAsyncResult *async_result, gpointer user_data)
+test_query_standard_feed_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
 {
 	GDataFeed *feed;
 	GError *error = NULL;
@@ -130,14 +131,13 @@ test_query_standard_feed_async_cb (GDataService *service, GAsyncResult *async_re
 }
 
 static void
-test_query_standard_feed_async (void)
+test_query_standard_feed_async (GDataService *service)
 {
-	g_assert (service != NULL);
+	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
 
 	gdata_youtube_service_query_standard_feed_async (GDATA_YOUTUBE_SERVICE (service), GDATA_YOUTUBE_TOP_RATED_FEED, NULL,
-							 NULL, NULL, NULL, (GAsyncReadyCallback) test_query_standard_feed_async_cb, NULL);
+							 NULL, NULL, NULL, (GAsyncReadyCallback) test_query_standard_feed_async_cb, main_loop);
 
-	main_loop = g_main_loop_new (NULL, TRUE);
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
 }
@@ -148,7 +148,7 @@ get_video_for_related (void)
 	GDataYouTubeVideo *video;
 	GError *error = NULL;
 
-	video = gdata_youtube_video_new_from_xml (
+	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
 		"<entry xmlns='http://www.w3.org/2005/Atom' "
 			"xmlns:media='http://search.yahoo.com/mrss/' "
 			"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
@@ -170,10 +170,10 @@ get_video_for_related (void)
 			"<category scheme='http://gdata.youtube.com/schemas/2007/keywords.cat' term='half-life'/>"
 			"<title type='text'>Escape From City 17 - Part One</title>"
 			"<content type='text'>Directed by The Purchase Brothers. *snip*</content>"
-			"<link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=q1UPMEmCqZo'/>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.youtube.com/watch?v=q1UPMEmCqZo'/>"
 			"<link rel='http://gdata.youtube.com/schemas/2007#video.related' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/q1UPMEmCqZo/related'/>"
 			"<link rel='http://gdata.youtube.com/schemas/2007#mobile' type='text/html' href='http://m.youtube.com/details?v=q1UPMEmCqZo'/>"
-			"<link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/standardfeeds/top_rated/v/q1UPMEmCqZo'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/standardfeeds/top_rated/v/q1UPMEmCqZo'/>"
 			"<author>"
 				"<name>PurchaseBrothers</name>"
 				"<uri>http://gdata.youtube.com/feeds/api/users/purchasebrothers</uri>"
@@ -203,7 +203,7 @@ get_video_for_related (void)
 			"<gd:comments>"
 				"<gd:feedLink href='http://gdata.youtube.com/feeds/api/videos/q1UPMEmCqZo/comments' countHint='13021'/>"
 			"</gd:comments>"
-		"</entry>", -1, &error);
+		"</entry>", -1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
 	g_clear_error (&error);
@@ -212,14 +212,12 @@ get_video_for_related (void)
 }
 
 static void
-test_query_related (void)
+test_query_related (GDataService *service)
 {
 	GDataFeed *feed;
 	GDataYouTubeVideo *video;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
 	video = get_video_for_related ();
 	feed = gdata_youtube_service_query_related (GDATA_YOUTUBE_SERVICE (service), video, NULL, NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
@@ -233,7 +231,7 @@ test_query_related (void)
 }
 
 static void
-test_query_related_async_cb (GDataService *service, GAsyncResult *async_result, gpointer user_data)
+test_query_related_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
 {
 	GDataFeed *feed;
 	GError *error = NULL;
@@ -250,24 +248,22 @@ test_query_related_async_cb (GDataService *service, GAsyncResult *async_result,
 }
 
 static void
-test_query_related_async (void)
+test_query_related_async (GDataService *service)
 {
 	GDataYouTubeVideo *video;
-
-	g_assert (service != NULL);
+	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
 
 	video = get_video_for_related ();
 	gdata_youtube_service_query_related_async (GDATA_YOUTUBE_SERVICE (service), video, NULL, NULL, NULL,
-						   NULL, (GAsyncReadyCallback) test_query_related_async_cb, NULL);
+						   NULL, (GAsyncReadyCallback) test_query_related_async_cb, main_loop);
 	g_object_unref (video);
 
-	main_loop = g_main_loop_new (NULL, TRUE);
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
 }
 
 static void
-test_upload_simple (void)
+test_upload_simple (GDataService *service)
 {
 	GDataYouTubeVideo *video, *new_video;
 	GDataMediaCategory *category;
@@ -275,8 +271,6 @@ test_upload_simple (void)
 	gchar *xml;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
 	video = gdata_youtube_video_new (NULL);
 
 	gdata_entry_set_title (GDATA_ENTRY (video), "Bad Wedding Toast");
@@ -284,10 +278,11 @@ test_upload_simple (void)
 	gdata_youtube_video_set_description (video, "I gave a bad toast at my friend's wedding.");
 	category = gdata_media_category_new ("People", NULL, "http://gdata.youtube.com/schemas/2007/categories.cat");
 	gdata_youtube_video_set_category (video, category);
+	g_object_unref (category);
 	gdata_youtube_video_set_keywords (video, "toast, wedding");
 
 	/* Check the XML */
-	xml = gdata_entry_get_xml (GDATA_ENTRY (video));
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
 	g_assert_cmpstr (xml, ==,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
@@ -304,7 +299,7 @@ test_upload_simple (void)
 	g_free (xml);
 
 	/* TODO: fix the path */
-	video_file = g_file_new_for_path ("/home/philip/Development/libgdata/gdata/tests/sample.ogg");
+	video_file = g_file_new_for_path (TEST_FILE_DIR "sample.ogg");
 
 	/* Upload the video */
 	new_video = gdata_youtube_service_upload_video (GDATA_YOUTUBE_SERVICE (service), video, video_file, NULL, &error);
@@ -320,13 +315,13 @@ test_upload_simple (void)
 }
 
 static void
-test_parsing_app_control (void)
+test_parsing_app_control (GDataService *service)
 {
 	GDataYouTubeVideo *video;
 	GDataYouTubeState *state;
 	GError *error = NULL;
 
-	video = gdata_youtube_video_new_from_xml (
+	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
 		"<entry xmlns='http://www.w3.org/2005/Atom' "
 			"xmlns:media='http://search.yahoo.com/mrss/' "
 			"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
@@ -341,8 +336,8 @@ test_parsing_app_control (void)
 			"</app:control>"
 			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
 			"<title>Judas Priest - Painkiller</title>"
-			"<link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
-			"<link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
 			"<author>"
 				"<name>eluves</name>"
 				"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
@@ -352,7 +347,7 @@ test_parsing_app_control (void)
 				"<media:credit role='uploader' scheme='urn:youtube'>eluves</media:credit>"
 				"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
 			"</media:group>"
-		"</entry>", -1, &error);
+		"</entry>", -1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
 	g_clear_error (&error);
@@ -361,10 +356,10 @@ test_parsing_app_control (void)
 	g_assert (gdata_youtube_video_is_draft (video) == TRUE);
 
 	state = gdata_youtube_video_get_state (video);
-	g_assert_cmpstr (state->name, ==, "blacklisted");
-	g_assert_cmpstr (state->message, ==, "This video is not available in your country");
-	g_assert (state->reason_code == NULL);
-	g_assert (state->help_uri == NULL);
+	g_assert_cmpstr (gdata_youtube_state_get_name (state), ==, "blacklisted");
+	g_assert_cmpstr (gdata_youtube_state_get_message (state), ==, "This video is not available in your country");
+	g_assert (gdata_youtube_state_get_reason_code (state) == NULL);
+	g_assert (gdata_youtube_state_get_help_uri (state) == NULL);
 
 	/* TODO: more tests on entry properties */
 
@@ -372,16 +367,16 @@ test_parsing_app_control (void)
 }
 
 static void
-test_parsing_yt_recorded (void)
+test_parsing_yt_recorded (GDataService *service)
 {
 	GDataYouTubeVideo *video;
 	GTimeVal recorded;
 	gchar *xml;
 	GError *error = NULL;
 
-	video = gdata_youtube_video_new_from_xml (
+	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
 		"<entry xmlns='http://www.w3.org/2005/Atom' "
-			"xmlns:media='http://search.yahoo.com/mrss/' "
+			"xmlns:media='http://video.search.yahoo.com/mrss' "
 			"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
 			"xmlns:gd='http://schemas.google.com/g/2005' "
 			"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
@@ -390,8 +385,8 @@ test_parsing_yt_recorded (void)
 			"<updated>2009-03-23T12:46:58.000Z</updated>"
 			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
 			"<title>Judas Priest - Painkiller</title>"
-			"<link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
-			"<link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
 			"<author>"
 				"<name>eluves</name>"
 				"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
@@ -402,7 +397,7 @@ test_parsing_yt_recorded (void)
 				"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
 			"</media:group>"
 			"<yt:recorded>2003-08-03</yt:recorded>"
-		"</entry>", -1, &error);
+		"</entry>", -1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
 	g_clear_error (&error);
@@ -417,29 +412,33 @@ test_parsing_yt_recorded (void)
 	gdata_youtube_video_set_recorded (video, &recorded);
 
 	/* Check the XML */
-	xml = gdata_entry_get_xml (GDATA_ENTRY (video));
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
 	g_assert_cmpstr (xml, ==,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
-				"xmlns:media='http://search.yahoo.com/mrss/' "
+				"xmlns:media='http://video.search.yahoo.com/mrss' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
 				"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+				"xmlns:app='http://www.w3.org/2007/app' "
 				"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
 				"<title type='text'>Judas Priest - Painkiller</title>"
 				"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
 				"<updated>2009-03-23T12:46:58Z</updated>"
 				"<published>2006-05-16T14:06:37Z</published>"
 				"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
-				"<link href='http://www.youtube.com/watch?v=JAagedeKdcQ' rel='alternate' type='text/html'/>"
-				"<link href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo' rel='self' type='application/atom+xml'/>"
+				"<link href='http://www.youtube.com/watch?v=JAagedeKdcQ' rel='http://www.iana.org/assignments/relation/alternate' type='text/html'/>"
+				"<link href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo' rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml'/>"
 				"<author>"
 					"<name>eluves</name>"
 					"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
 				"</author>"
 				"<media:group>"
-					"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
+					"<media:category scheme='http://gdata.youtube.com/schemas/2007/categories.cat' label='Music'>Music</media:category>"
 					"<media:title type='plain'>Judas Priest - Painkiller</media:title>"
 				"</media:group>"
 				"<yt:recorded>2005-10-02</yt:recorded>"
+				"<app:control>"
+					"<app:draft>no</app:draft>"
+				"</app:control>"
 			 "</entry>");
 	g_free (xml);
 
@@ -448,14 +447,14 @@ test_parsing_yt_recorded (void)
 	g_object_unref (video);
 }
 
-static void
+/*static void
 test_parsing_comments_feed_link (void)
 {
 	GDataYouTubeVideo *video;
 	GDataGDFeedLink *feed_link;
 	GError *error = NULL;
 
-	video = gdata_youtube_video_new_from_xml (
+	video = gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
 		"<entry xmlns='http://www.w3.org/2005/Atom' "
 			"xmlns:media='http://search.yahoo.com/mrss/' "
 			"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
@@ -466,8 +465,8 @@ test_parsing_comments_feed_link (void)
 			"<updated>2009-03-23T12:46:58.000Z</updated>"
 			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
 			"<title>Judas Priest - Painkiller</title>"
-			"<link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
-			"<link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
 			"<author>"
 				"<name>eluves</name>"
 				"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
@@ -485,7 +484,7 @@ test_parsing_comments_feed_link (void)
 	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
 	g_clear_error (&error);
 
-	/* Test the feed link */
+	* Test the feed link *
 	feed_link = gdata_youtube_video_get_comments_feed_link (video);
 	g_assert (feed_link != NULL);
 	g_assert (feed_link->rel == NULL);
@@ -493,13 +492,13 @@ test_parsing_comments_feed_link (void)
 	g_assert_cmpuint (feed_link->count_hint, ==, 13021);
 	g_assert (feed_link->read_only == FALSE);
 
-	/* TODO: more tests on entry properties */
+	* TODO: more tests on entry properties *
 
 	g_object_unref (video);
-}
+}*/
 
 static void
-test_query_uri (void)
+test_query_uri (GDataService *service)
 {
 	gdouble latitude, longitude, radius;
 	gboolean has_location;
@@ -519,7 +518,7 @@ test_query_uri (void)
 	g_assert (has_location == TRUE);
 
 	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com");
-	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=all_time&safeSearch=none&format=1&location=45.013640,-97.123560!&location-radius=112.500000m");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=all_time&safeSearch=none&format=1&location=45.013640000000002,-97.123559999999998!&location-radius=112.5m");
 	g_free (query_uri);
 
 	gdata_youtube_query_set_location (query, G_MAXDOUBLE, 0.6672, 52.8, TRUE);
@@ -577,35 +576,111 @@ test_query_uri (void)
 	g_object_unref (query);
 }
 
+static void
+test_query_single (GDataService *service)
+{
+	GDataYouTubeVideo *video;
+	GError *error = NULL;
+
+	video = gdata_youtube_service_query_single_video (GDATA_YOUTUBE_SERVICE (service), NULL, "_LeQuMpwbW4", NULL, &error);
+
+	g_assert_no_error (error);
+	g_assert (video != NULL);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_assert_cmpstr (gdata_youtube_video_get_video_id (video), ==, "_LeQuMpwbW4");
+	g_assert_cmpstr (gdata_entry_get_id (GDATA_ENTRY (video)), ==, "tag:youtube.com,2008:video:_LeQuMpwbW4");
+	g_clear_error (&error);
+
+	g_object_unref (video);
+}
+
+static void
+test_query_single_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GDataYouTubeVideo *video;
+	GError *error = NULL;
+
+	video = gdata_youtube_service_query_single_video_finish (GDATA_YOUTUBE_SERVICE (service), async_result, &error);
+
+	g_assert_no_error (error);
+	g_assert (video != NULL);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_assert_cmpstr (gdata_youtube_video_get_video_id (video), ==, "_LeQuMpwbW4");
+	g_assert_cmpstr (gdata_entry_get_id (GDATA_ENTRY (video)), ==, "tag:youtube.com,2008:video:_LeQuMpwbW4");
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+	g_object_unref (video);
+}
+
+static void
+test_query_single_async (GDataService *service)
+{
+	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
+
+	gdata_youtube_service_query_single_video_async (GDATA_YOUTUBE_SERVICE (service), NULL, "_LeQuMpwbW4", NULL,
+							(GAsyncReadyCallback) test_query_single_async_cb, main_loop);
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+}
+
+static void
+test_parsing_video_id_from_uri (void)
+{
+	gchar *video_id;
+
+	video_id = gdata_youtube_video_get_video_id_from_uri ("http://www.youtube.com/watch?v=BH_vwsyCrTc&feature=featured");
+	g_assert_cmpstr (video_id, ==, "BH_vwsyCrTc");
+	g_free (video_id);
+
+	video_id = gdata_youtube_video_get_video_id_from_uri ("http://www.youtube.es/watch?v=foo");
+	g_assert_cmpstr (video_id, ==, "foo");
+	g_free (video_id);
+
+	video_id = gdata_youtube_video_get_video_id_from_uri ("http://foobar.com/watch?v=foo");
+	g_assert (video_id == NULL);
+
+	video_id = gdata_youtube_video_get_video_id_from_uri ("http://foobar.com/not/real");
+	g_assert (video_id == NULL);
+}
+
 int
 main (int argc, char *argv[])
 {
 	gint retval;
+	GDataService *service;
 
 	g_type_init ();
 	g_thread_init (NULL);
 	g_test_init (&argc, &argv, NULL);
 	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
 
+	service = GDATA_SERVICE (gdata_youtube_service_new (DEVELOPER_KEY, CLIENT_ID));
+	gdata_service_authenticate (service, YT_USERNAME, YT_PASSWORD, NULL, NULL);
+
 	g_test_add_func ("/youtube/authentication", test_authentication);
 	if (g_test_thorough () == TRUE)
 		g_test_add_func ("/youtube/authentication_async", test_authentication_async);
-	g_test_add_func ("/youtube/query/standard_feed", test_query_standard_feed);
+	g_test_add_data_func ("/youtube/query/standard_feed", service, test_query_standard_feed);
 	if (g_test_thorough () == TRUE)
-		g_test_add_func ("/youtube/query/standard_feed_async", test_query_standard_feed_async);
-	g_test_add_func ("/youtube/query/related", test_query_related);
+		g_test_add_data_func ("/youtube/query/standard_feed_async", service, test_query_standard_feed_async);
+	g_test_add_data_func ("/youtube/query/related", service, test_query_related);
 	if (g_test_thorough () == TRUE)
-		g_test_add_func ("/youtube/query/related_async", test_query_related_async);
+		g_test_add_data_func ("/youtube/query/related_async", service, test_query_related_async);
 	if (g_test_slow () == TRUE)
-		g_test_add_func ("/youtube/upload/simple", test_upload_simple);
-	g_test_add_func ("/youtube/parsing/app:control", test_parsing_app_control);
-	g_test_add_func ("/youtube/parsing/comments/feedLink", test_parsing_comments_feed_link);
-	g_test_add_func ("/youtube/parsing/yt:recorded", test_parsing_yt_recorded);
-	g_test_add_func ("/youtube/query/uri", test_query_uri);
+		g_test_add_data_func ("/youtube/upload/simple", service, test_upload_simple);
+	g_test_add_data_func ("/youtube/parsing/app:control", service, test_parsing_app_control);
+	/*g_test_add_func ("/youtube/parsing/comments/feedLink", test_parsing_comments_feed_link);*/
+	g_test_add_data_func ("/youtube/parsing/yt:recorded", service, test_parsing_yt_recorded);
+	g_test_add_data_func ("/youtube/query/uri", service, test_query_uri);
+	g_test_add_data_func ("/youtube/query/single", service, test_query_single);
+	if (g_test_slow () == TRUE)
+		g_test_add_data_func ("/youtube/query/single_async", service, test_query_single_async);
+	g_test_add_func ("/youtube/parsing/video_id_from_uri", test_parsing_video_id_from_uri);
 
 	retval = g_test_run ();
-	if (service != NULL)
-		g_object_unref (service);
+	g_object_unref (service);
 
 	return retval;
 }
